@@ -21,6 +21,8 @@ from maps4fs.generator.component.dem import DEM
 from maps4fs.generator.component.texture import Texture
 from maps4fs.generator.settings import Parameters
 
+SEGMENT_LENGTH = 2
+
 
 class Background(MeshComponent, ImageComponent):
     """Component for creating 3D obj files based on DEM data around the map.
@@ -97,6 +99,10 @@ class Background(MeshComponent, ImageComponent):
         cutted_dem_path = self.save_map_dem(self.output_path)
         shutil.copyfile(self.output_path, self.not_substracted_path)
         self.save_map_dem(self.output_path, save_path=self.not_resized_path)
+
+        if self.map.background_settings.flatten_roads:
+            self.flatten_roads()
+
         if self.game.additional_dem_name is not None:
             self.make_copy(cutted_dem_path, self.game.additional_dem_name)
 
@@ -521,6 +527,10 @@ class Background(MeshComponent, ImageComponent):
             int(self.map.dem_settings.water_depth * z_scaling_factor),
         )
 
+        dem_image = self.blur_edges_by_mask(
+            dem_image, water_resources_image, smaller_kernel=3, iterations=5, bigger_kernel=5  # type: ignore
+        )
+
         # Save the modified dem_image back to the output path
         cv2.imwrite(self.output_path, dem_image)
         self.logger.debug("Water depth subtracted from DEM data: %s", self.output_path)
@@ -542,41 +552,28 @@ class Background(MeshComponent, ImageComponent):
         It creates polygons from the polylines, fits them into the map bounds, and generates a mesh.
         """
         self.logger.debug("Starting line-based water generation...")
-
-        water_polylines = self.get_infolayer_data(Parameters.BACKGROUND, Parameters.WATER_POLYLINES)
+        water_polygons = self.get_infolayer_data(Parameters.BACKGROUND, Parameters.WATER)
         self.logger.debug(
-            "Found %s water polylines in background info layer.", len(water_polylines)  # type: ignore
+            "Found %s water polygons in background info layer.", len(water_polygons)  # type: ignore
         )
-        if not water_polylines:
-            self.logger.warning("No water polylines found in background info layer.")
+        if not water_polygons:
+            self.logger.warning("No water polygons found in background info layer.")
             return
 
         polygons: list[shapely.Polygon] = []
-        for polyline in water_polylines:
-            points = polyline["points"]
-            width = polyline["width"]
-            if not points or len(points) < 2:
-                self.logger.warning("Skipping polyline with insufficient points: %s", polyline)
+        for polygon_points in water_polygons:
+            if not polygon_points or len(polygon_points) < 2:
+                self.logger.warning("Skipping polygon with insufficient points...")
                 continue
 
-            # Create a shapely LineString from the points
-            line = shapely.geometry.LineString(points)
-            # Create a buffer around the line to create a polygon
-            if width <= 0:
-                self.logger.warning("Skipping polyline with non-positive width: %s", polyline)
+            polygon = shapely.Polygon(polygon_points)
+
+            if polygon.is_empty or not polygon.is_valid:
+                self.logger.warning("Skipping empty or invalid polygon...")
                 continue
 
-            polygon = line.buffer(
-                width + Parameters.WATER_ADD_WIDTH, cap_style=shapely.geometry.CAP_STYLE.square
-            )
-            if polygon.is_empty:
-                self.logger.warning("Skipping empty polygon created from polyline: %s", polyline)
-                continue
-
-            # Ensure the polygon is valid and not empty
-            if not polygon.is_valid:
-                self.logger.warning("Invalid polygon created from polyline, skipping: %s", polyline)
-                continue
+            # Make Polygon a little bit bigger to hide under the terrain when creating water planes.
+            polygon = polygon.buffer(Parameters.WATER_ADD_WIDTH, resolution=4)
 
             polygons.append(polygon)
 
@@ -736,3 +733,94 @@ class Background(MeshComponent, ImageComponent):
         self.assets.water_mesh = elevated_save_path
 
         self.plane_from_np(elevated_water, elevated_save_path, include_zeros=False)
+
+    def flatten_roads(self) -> None:
+        """Flattens the roads in the DEM data by averaging the height values along the road polylines."""
+        if not self.not_resized_path or not os.path.isfile(self.not_resized_path):
+            self.logger.warning("No DEM data found for flattening roads.")
+            return
+
+        dem_image = cv2.imread(self.not_resized_path, cv2.IMREAD_UNCHANGED)
+        if dem_image is None:
+            self.logger.warning("Failed to read DEM data.")
+            return
+
+        roads_polylines = self.get_infolayer_data(Parameters.TEXTURES, Parameters.ROADS_POLYLINES)
+        if not roads_polylines:
+            self.logger.warning("No roads polylines found in textures info layer.")
+            return
+
+        self.logger.debug("Found %s roads polylines in textures info layer.", len(roads_polylines))
+
+        full_mask = np.zeros(dem_image.shape, dtype=np.uint8)
+
+        for road_polyline in tqdm(roads_polylines, desc="Flattening roads", unit="road"):
+            points = road_polyline.get("points")
+            width = road_polyline.get("width")
+            if not points or not width:
+                self.logger.warning("Skipping road with insufficient data: %s", road_polyline)
+                continue
+
+            try:
+                fitted_road = self.fit_object_into_bounds(
+                    linestring_points=points, angle=self.rotation
+                )
+            except ValueError as e:
+                self.logger.debug(
+                    "Road polyline could not be fitted into the map bounds with error: %s",
+                    e,
+                )
+                continue
+
+            polyline = shapely.LineString(fitted_road)
+
+            total_length = polyline.length
+            self.logger.debug("Total length of the road polyline: %s", total_length)
+
+            current_distance = 0
+            segments: list[shapely.LineString] = []
+            while current_distance < total_length:
+                start_point = polyline.interpolate(current_distance)
+                end_distance = min(current_distance + SEGMENT_LENGTH, total_length)
+                end_point = polyline.interpolate(end_distance)
+
+                # Create a small segment LineString
+                segment = shapely.LineString([start_point, end_point])
+                segments.append(segment)
+                current_distance += SEGMENT_LENGTH
+
+            self.logger.debug("Number of segments created: %s", len(segments))
+
+            road_polygons: list[shapely.Polygon] = []
+            for segment in segments:
+                polygon = segment.buffer(
+                    width * 2, resolution=4, cap_style="flat", join_style="mitre"
+                )
+                road_polygons.append(polygon)
+
+            for polygon in road_polygons:
+                polygon_points = polygon.exterior.coords
+                road_np = self.polygon_points_to_np(polygon_points)
+                mask = np.zeros(dem_image.shape, dtype=np.uint8)
+
+                try:
+                    cv2.fillPoly(mask, [road_np], 255)  # type: ignore
+                except Exception as e:
+                    self.logger.debug("Could not create mask for road with error: %s", e)
+                    continue
+
+                mean_value = cv2.mean(dem_image, mask=mask)[0]  # type: ignore
+                dem_image[mask == 255] = mean_value
+                full_mask[mask == 255] = 255
+
+        main_dem_path = self.game.dem_file_path(self.map_directory)
+        dem_image = self.blur_by_mask(dem_image, full_mask)
+        dem_image = self.blur_edges_by_mask(dem_image, full_mask)
+
+        output_size = dem_image.shape[0] + 1
+        resized_dem = cv2.resize(
+            dem_image, (output_size, output_size), interpolation=cv2.INTER_NEAREST
+        )
+
+        cv2.imwrite(main_dem_path, resized_dem)
+        self.logger.debug("Flattened roads saved to DEM file: %s", main_dem_path)
