@@ -19,6 +19,7 @@ class Parameters:
     FOREST = "forest"
     ROADS_POLYLINES = "roads_polylines"
     WATER_POLYLINES = "water_polylines"
+    WATER = "water"
     FARMYARDS = "farmyards"
 
     PREVIEW_MAXIMUM_SIZE = 2048
@@ -39,6 +40,8 @@ class Parameters:
 
     WATER_ADD_WIDTH = 2
 
+    HEIGHT_SCALE = "heightScale"
+
 
 class SharedSettings(BaseModel):
     """Represents the shared settings for all components."""
@@ -160,12 +163,14 @@ class BackgroundSettings(SettingsModel):
         water_blurriness (int): blurriness of the water.
         remove_center (bool): remove the center of the background terrain.
             It will be used to remove the center of the map where the player starts.
+        flatten_roads (bool): if True, roads will be flattened in the DEM data.
     """
 
     generate_background: bool = False
     generate_water: bool = False
     water_blurriness: int = 20
     remove_center: bool = True
+    flatten_roads: bool = False
 
 
 class GRLESettings(SettingsModel):
@@ -200,6 +205,7 @@ class I3DSettings(SettingsModel):
     Attributes:
         add_trees (bool): add trees to the map.
         forest_density (int): density of the forest (distance between trees).
+        tree_limit (int): maximum number of trees to be added to the map.
         trees_relative_shift (int): relative shift of the trees.
         spline_density (int): the number of extra points that will be added between each two
             existing points.
@@ -209,6 +215,7 @@ class I3DSettings(SettingsModel):
 
     add_trees: bool = True
     forest_density: int = 10
+    tree_limit: int = 0
     trees_relative_shift: int = 20
 
     spline_density: int = 2
